﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Graphics
{
    /// <summary>
    /// Represents an image to be displayed on the console.  Can be either a still or an animation.
    /// </summary>
    public abstract class Sprite
    {
        public ConsoleColor foregroundColor = Graphics.Defaults.defaultForeground;
        public ConsoleColor backgroundColor = Graphics.Defaults.defaultBackground;
        public char transparencyKey = Graphics.Defaults.defaultTransparencyKey;

        public bool hasTransparency = false;
        public bool disabled = false;
        public bool hasColor = false;

        public string[] image;
        public string[] fcolor;
        public string[] bcolor;
        public string[] solid;

        public int xP, yP;

        //Custom Variables for Game
        public int health = 5;
        public bool flipped = false;
        public string _path;
        public int points = 25;
        public int speed = 5;
        public int frescared = 0;

        /// <summary>
        /// This method is called right before the sprite is to be printed to the screen/buffer
        /// </summary>
        public abstract void onPrint();

        /// <summary>
        /// Detects collision (overlap) between two Sprites
        /// </summary>
        /// <param name="s">Other Sprite to detect collision with this Sprite</param>
        /// <returns>True if collision occurs, False otherwise</returns>
        public bool collision(Sprite s)
        {
            //For each position on the board for Sprite1 (s1x, s1y)
            if (solid != null && s.solid != null)
                for (int s1x = xP; s1x < xP + solid[0].Length; s1x++)
                    for (int s1y = yP; s1y < yP + solid.Length; s1y++)
                        if (solid[s1y - yP][s1x - xP] == 'X')
                            if (s1x - s.xP > 0 && s1x - s.xP < s.solid[0].Length && s1y - s.yP > 0 && s1y - s.yP < s.solid.Length)
                                if (s.solid[s1y - s.yP][s1x - s.xP] == 'X')
                                    return true;

            return false;
        }

        /// <summary>
        /// Detects collision (overlap) between this Sprite and a collection of other Sprites
        /// </summary>
        /// <param name="collection">Collection of other Sprites</param>
        /// <returns>null - no collision, or the first Sprite that a collision was detected</returns>
        public Sprite collision(ICollection<Sprite> collection)
        {
            foreach (Sprite s in collection)
            {
                if (!s.Equals(this))
                    if (collision(s))
                        return s;
            }

            return null;
        }

        public void changeImage(bool confirm, string path)
        {
            if (confirm)
            {
                flipped = !flipped;
                _path = path.Substring(0, path.Length - 2);

                string[] entireFile = Graphics.Defaults.fileToStringArray(path);

                //Set params
                if (entireFile[0].Contains("KEY"))
                {
                    hasTransparency = true;
                    transparencyKey = entireFile[0].ToCharArray()[entireFile[0].IndexOf("KEY") + 4];
                }
                if (entireFile[0].Contains("BCOLOR"))
                {
                    hasColor = true;
                    backgroundColor = Graphics.Defaults.getColor(entireFile[0].ToCharArray()[entireFile[0].IndexOf("BCOLOR") + 7]);
                }
                if (entireFile[0].Contains("FCOLOR"))
                {
                    hasColor = true;
                    foregroundColor = Graphics.Defaults.getColor(entireFile[0].ToCharArray()[entireFile[0].IndexOf("FCOLOR") + 7]);
                }

                //Set Image
                image = new string[entireFile.Length - 1];
                for (int i = 0; i < image.Length; i++)
                    image[i] = entireFile[i + 1];

                //Get collision data
                entireFile = Graphics.Defaults.fileToStringArray(path + ".col");
                if (entireFile != null)
                {
                    solid = new string[image.Length];
                    for (int i = 0; i < image.Length; i++)
                        solid[i] = entireFile[i];
                }

                //Get bcolor data
                entireFile = Graphics.Defaults.fileToStringArray(path + ".b");
                if (entireFile != null)
                {
                    hasColor = true;
                    bcolor = new string[image.Length];
                    for (int i = 0; i < image.Length; i++)
                        bcolor[i] = entireFile[i];
                }

                //Get fcolor data
                entireFile = Graphics.Defaults.fileToStringArray(path + ".f");
                if (entireFile != null)
                {
                    hasColor = true;
                    fcolor = new string[image.Length];
                    for (int i = 0; i < image.Length; i++)
                        fcolor[i] = entireFile[i];
                }
            }
        }
    }
}
